# Generated by Django 2.2.3 on 2021-02-21 02:58

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('epayco_django', '0004_add_length_to_transaction_fields'),
    ]

    operations = [
        migrations.AlterField(
            model_name='paymentconfirmation',
            name='customer_doctype',
            field=models.CharField(max_length=12),
        ),
        migrations.AlterField(
            model_name='paymentconfirmation',
            name='customer_document',
            field=models.CharField(max_length=64),
        ),
    ]



import json

from django.http import JsonResponse, HttpResponseRedirect
from django.urls import reverse
from django.utils.decorators import method_decorator
from django.views.decorators.csrf import csrf_exempt
from django.views.generic import View, TemplateView

from .models import PaymentConfirmation
from .utils import validate_response_code


@method_decorator(csrf_exempt, 'dispatch')
class ConfirmationView(View):

    def post(self, request, *args, **kwargs):
        data = {}
        for k, v in request.POST.items():
            if k in ('x_id_factura', 'x_respuesta',
                     'x_fecha_transaccion', 'x_cod_respuesta'):
                # Ignore fields in both English and Spanish
                continue
            data[k.replace('x_', '')] = v
        # TODO: Validate name consistency across all abstract models to prevent things like: id_user and customer_id.
        # Invert invoice name to keep the names consistent
        data['invoice_id'] = data['id_invoice']
        del data['id_invoice']

        # Get boolean fron the test_request attribute
        data['test_request'] = data['test_request'] == 'TRUE'
        data['raw'] = json.dumps(request.POST)

        # The AbstractFlagSegment's model's save method does the flagging validations.
        item = PaymentConfirmation.objects.create(**data)
        return JsonResponse({'flag': item.is_flagged}, status=200)


@method_decorator(csrf_exempt, 'dispatch')
class ResponseValidationView(TemplateView):
    template_name = 'simple_payment_response.html'

    def get_context_data(self, **kwargs):
        context = super().get_context_data()
        ref = self.request.GET.get('ref_payco')
        validation = validate_response_code(ref, self.request)
        context['payment'] = validation
        return context

    def post(self, request, **kwargs):
        ref = request.POST.get('x_ref_payco', None)
        if ref is None:
            return JsonResponse({'ref': ['This field is required.']}, status=400)
        validate_response_code(ref, request)
        return HttpResponseRedirect(reverse('epayco_response_validation') + '?ref_payco={}'.format(ref))



# Generated by Django 2.2.3 on 2020-09-10 00:56

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('epayco_django', '0002_transaction_id_max_length'),
    ]

    operations = [
        migrations.AlterField(
            model_name='paymentconfirmation',
            name='response_reason_text',
            field=models.CharField(max_length=256),
        ),
    ]



from django import template

from ..settings import epayco_settings

register = template.Library()


@register.inclusion_tag('epayco_web_checkout_button.html')
def render_checkout(amount, name, tax=0, tax_base=0, description='', currency='COP', country_code='co',
                    external=True, extra1='', extra2='', extra3='', extra4='', extra5='', extra6='', extra7='',
                    extra8='', extra9='', extra10='', same_site=True, request=None, response_url=None, **kwargs):
    if description == '':
        description = name

    if epayco_settings.CONFIRMATION_URL.startswith('http'):
        confirmation_url = epayco_settings.CONFIRMATION_URL
    else:
        confirmation_url = request.build_absolute_uri(epayco_settings.CONFIRMATION_URL)

    if epayco_settings.FORCE_HTTPS and 'https://' not in confirmation_url:
        confirmation_url = confirmation_url.replace('http://', 'https://')

    if not response_url:
        if epayco_settings.CONFIRMATION_URL.startswith('http'):
            response_url = epayco_settings.RESPONSE_URL
        else:
            response_url = request.build_absolute_uri(epayco_settings.RESPONSE_URL)

    if epayco_settings.FORCE_HTTPS and 'https://' not in response_url:
        response_url = response_url.replace('http://', 'https://')

    return {
        'public_key': epayco_settings.PUBLIC_KEY,
        'amount': amount,
        'tax': tax,
        'tax_base': tax_base,
        'name': name,
        'description': description,
        'currency': currency,
        'country_code': country_code,
        'test': '{}'.format(epayco_settings.TEST).lower(),
        'external': '{}'.format(external).lower(),
        'extra1': extra1,
        'extra2': extra2,
        'extra3': extra3,
        'extra4': extra4,
        'extra5': extra5,
        'extra6': extra6,
        'extra7': extra7,
        'extra8': extra8,
        'extra9': extra9,
        'extra10': extra10,
        'response_url': response_url,
        'confirmation_url': confirmation_url,
        'same_site': same_site,
    }



"""
    This module follows the approach Django Rest Framework settings.
    The 'settings.py' file might look like this:
    EPAYCO = {
        'PUBLIC_KEY': 'MY_PUBLIC_KEY',
        'PRIVATE_KEY': 'MY_PRIVATE_KEY',
        'P_KEY': 'MY_P_KEY',
        'TEST': True, # you probably want to test it first rigth?

        # Optional (ignore these if you want the default behaviour)
        'FORCE_HTTPS': False, # If you use https on your website you may want to enable this.
        'RESPONSE_URL': reverse('epayco_response'),
        'CONFIRMATION_URL': reverse('epayco_confirmation'),

        # Instead of reversing a project url you could just use any url instead:
        'CONFIRMATION_URL': 'https://yourdomain.com/epayco/confirmation',

        # And you can use your own image as a payment button
        'CHECKOUT_BUTTON_URL': 'https://mydomain.com/btns/pay_now_button.png'
    }
"""
from django.conf import settings
from django.core.exceptions import ImproperlyConfigured
from django.urls import reverse

USER_SETTINGS = getattr(settings, "EPAYCO", None)

if not isinstance(USER_SETTINGS, dict):
    raise ImproperlyConfigured('The ePayco setting is not properly set."')

# List of settings that have a default when a value is not provided by the user.
DEFAULTS = {
    'PUBLIC_KEY': None,
    'PRIVATE_KEY': None,
    'P_KEY': None,
    'TEST': False,
    'CONFIRMATION_URL': 'epayco_confirmation',
    'RESPONSE_URL': 'epayco_response_validation',
    'CHECKOUT_BUTTON_URL': 'https://369969691f476073508a-60bf0867add971908d4f26a64'
                           '519c2aa.ssl.cf5.rackcdn.com/btns/boton_carro_de_compras_epayco5.png',
    'FORCE_HTTPS': True,  # Activates HTTPS on requests
}

# List of settings that cannot be empty
MANDATORY = (
    'PUBLIC_KEY',
    'PRIVATE_KEY',
    'P_KEY',
)


class EpaycoSettings(object):
    """
        Settings object that allows accessing the ePayco settings as properties.
    """

    def __init__(self, user_settings=None, defaults=None, mandatory=None):
        self.user_settings = user_settings or {}
        self.defaults = defaults or {}
        self.mandatory = mandatory or {}

    def __getattr__(self, attr):
        if attr not in self.defaults.keys():
            raise AttributeError("Invalid ePayco setting: %r" % (attr))

        try:
            # Check if present in user settings
            val = self.user_settings[attr]
        except KeyError:
            # Fall back to defaults
            val = self.defaults[attr]

        if attr in ['CONFIRMATION_URL', 'RESPONSE_URL'] and not attr.startswith('https://'):
            val = reverse(val)

        self.validate_setting(attr, val)

        # Cache the result
        setattr(self, attr, val)

        return val

    def validate_setting(self, attr, val):
        if not val and attr in self.mandatory:
            raise ImproperlyConfigured("ePayco setting: '{}' is mandatory".format(attr))


epayco_settings = EpaycoSettings(USER_SETTINGS, DEFAULTS, MANDATORY)



# Generated by Django 2.2.8 on 2020-05-18 15:02

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('epayco_django', '0001_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='paymentconfirmation',
            name='extra1',
            field=models.CharField(blank=True, max_length=255),
        ),
        migrations.AlterField(
            model_name='paymentconfirmation',
            name='extra10',
            field=models.CharField(blank=True, max_length=255),
        ),
        migrations.AlterField(
            model_name='paymentconfirmation',
            name='extra2',
            field=models.CharField(blank=True, max_length=255),
        ),
        migrations.AlterField(
            model_name='paymentconfirmation',
            name='extra3',
            field=models.CharField(blank=True, max_length=255),
        ),
        migrations.AlterField(
            model_name='paymentconfirmation',
            name='extra4',
            field=models.CharField(blank=True, max_length=255),
        ),
        migrations.AlterField(
            model_name='paymentconfirmation',
            name='extra5',
            field=models.CharField(blank=True, max_length=255),
        ),
        migrations.AlterField(
            model_name='paymentconfirmation',
            name='extra6',
            field=models.CharField(blank=True, max_length=255),
        ),
        migrations.AlterField(
            model_name='paymentconfirmation',
            name='extra7',
            field=models.CharField(blank=True, max_length=255),
        ),
        migrations.AlterField(
            model_name='paymentconfirmation',
            name='extra8',
            field=models.CharField(blank=True, max_length=255),
        ),
        migrations.AlterField(
            model_name='paymentconfirmation',
            name='extra9',
            field=models.CharField(blank=True, max_length=255),
        ),
        migrations.AlterField(
            model_name='paymentconfirmation',
            name='flag_code',
            field=models.CharField(blank=True, choices=[('1001', 'Duplicate Transaction'), ('1002', 'Invalid Sign'), ('1003', 'Test transacción on non test environment')], default='', max_length=4),
        ),
        migrations.AlterField(
            model_name='paymentconfirmation',
            name='flag_info',
            field=models.CharField(blank=True, default='', max_length=100),
        ),
        migrations.AlterField(
            model_name='paymentconfirmation',
            name='transaction_id',
            field=models.CharField(max_length=64),
        ),
    ]



import hashlib

import epaycosdk.epayco as Epayco
import requests

from .settings import epayco_settings


def get_signature(cust_id_client, p_key, ref_payco, transaction_id, amount, currency_code):
    """
        Genera el signature requerido por ePayco para verificar una confirmación.
    """
    signature = '{}^{}^{}^{}^{}^{}'.format(cust_id_client, p_key, ref_payco,
                                           transaction_id, amount, currency_code)
    return hashlib.sha256(signature.encode('utf')).hexdigest()


def validate_response_code(ref_payco, request=None):
    """
    Verifica el ref_payco. Funciona con los 2 tipos de ref_payco.

    :param ref_payco:
    :param request:
    :return:
    """

    if not ref_payco:
        return {'valid_ref': False}

    from .models import PaymentConfirmation
    options = {'apiKey': epayco_settings.PUBLIC_KEY,
               'privateKey': epayco_settings.PRIVATE_KEY,
               'test': epayco_settings.TEST,
               'lenguage': 'ES'}

    response = None
    if not ref_payco.isdigit():
        response = requests.get('https://secure.epayco.co/validation/v1/reference/{}'.format(ref_payco))
        try:
            response = response.json()
        except:
            return {'valid_ref': False}
        ref_payco = str(response['data']['x_ref_payco'])

    qs = PaymentConfirmation.objects.filter(ref_payco__iexact=ref_payco).exclude(cod_transaction_state=3)
    if not qs.exists():
        if not response:
            epayco = Epayco.Epayco(options)
            response = epayco.cash.get(ref_payco)

        # Validate the reference
        if response['success'] == True:
            if epayco_settings.CONFIRMATION_URL.startswith('http'):
                url = epayco_settings.CONFIRMATION_URL
            else:
                url = '{}://{}{}'.format('https' if epayco_settings.FORCE_HTTPS or request.is_secure() else 'http',
                                         request.get_host(), epayco_settings.CONFIRMATION_URL)
            r2 = requests.post(url, data=response['data'])
            if r2.status_code == 405:
                raise Exception('There seems the be an error reaching the confirmation URL.'
                                ' Please make sure you are making good use of the "FORCE_HTTPS" setting.')
            obj = PaymentConfirmation.objects.filter(ref_payco__iexact=ref_payco).last()
            if obj:
                return {'valid_ref': True, 'existed': False, 'flag': obj.is_flagged, 'obj': obj}
            else:
                return {''}
        return {'valid_ref': False}
    elif qs.filter(flag=False).exists():
        obj = qs.filter(flag=False).last()
        return {'valid_ref': True, 'existed': True, 'flag': False, 'obj': obj}
    else:
        obj = qs.last()
        return {'valid_ref': True, 'existed': True, 'flag': True, 'obj': obj}



# Generated by Django 2.2.3 on 2021-02-21 02:36

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('epayco_django', '0003_extend_response_reason_text_length'),
    ]

    operations = [
        migrations.AlterField(
            model_name='paymentconfirmation',
            name='approval_code',
            field=models.CharField(max_length=32),
        ),
        migrations.AlterField(
            model_name='paymentconfirmation',
            name='cod_response',
            field=models.CharField(max_length=32),
        ),
        migrations.AlterField(
            model_name='paymentconfirmation',
            name='cod_transaction_state',
            field=models.CharField(max_length=32),
        ),
        migrations.AlterField(
            model_name='paymentconfirmation',
            name='errorcode',
            field=models.CharField(max_length=32),
        ),
        migrations.AlterField(
            model_name='paymentconfirmation',
            name='franchise',
            field=models.CharField(max_length=32),
        ),
        migrations.AlterField(
            model_name='paymentconfirmation',
            name='response',
            field=models.CharField(max_length=32),
        ),
        migrations.AlterField(
            model_name='paymentconfirmation',
            name='transaction_state',
            field=models.CharField(max_length=32),
        ),
    ]
